package es.ucm.fdi.grasia.faerie.observer.internal.attributeConditionSet;

import java.net.URI;
import java.util.ArrayList;
import java.util.Map;

import org.apache.felix.scr.annotations.Component;

import es.ucm.fdi.grasia.faerie.core.AbstractRelationshipUpdaterComponent;
import es.ucm.fdi.grasia.faerie.core.Attribute;
import es.ucm.fdi.grasia.faerie.core.ContextElement;
import es.ucm.fdi.grasia.faerie.core.ContextEvent;
import es.ucm.fdi.grasia.faerie.core.ContextValue;
import es.ucm.fdi.grasia.faerie.core.ContextObserver;
import es.ucm.fdi.grasia.faerie.core.Entity;
import es.ucm.fdi.grasia.faerie.core.Relationship;
import es.ucm.fdi.grasia.faerie.core.model.IncludedEntities;
import es.ucm.fdi.grasia.faerie.core.util.Functions;
import es.ucm.fdi.grasia.faerie.model.base.AttributeCondition;
import es.ucm.fdi.grasia.faerie.model.base.AttributeConditionSet;
import org.apache.log4j.Logger;

@Component(inherit = true)
@SuppressWarnings({ "rawtypes", "unchecked" })
public class AttributeConditionSetObserver extends AbstractRelationshipUpdaterComponent {

	@Override
	public int handle(ContextEvent event) {
		return Functions.relationshipInstanceOfEntityClassAddedOrRemoved(AttributeConditionSet.class,
				IncludedEntities.class.getName(), event) ? ContextObserver.HANDLE_WRITE : 0;
	}

	@Override
	public void elementAdded(Relationship relationshipInstance, ContextEvent event) {
		this.putUpdatedRelationshipInstance(relationshipInstance);

		URI relInstanceId = relationshipInstance.getId();

		this.addRelationshipInstanceLinkedElement(relInstanceId, this.getEnvironmentReferencedEntities());

		this.updateRelationshipInstance(relInstanceId);
	}

	@Override
	public void relationshipInstanceLinkedElementsUpdated(Relationship relationshipInstance,
			Map<URI, ContextElement> linkedElements) {
		IncludedEntities queryIncludedEntities = (IncludedEntities) relationshipInstance;

		AttributeConditionSet attributeConditionSet = (AttributeConditionSet) queryIncludedEntities.getSourceEntity();

		Class<? extends Entity> entityType = attributeConditionSet.getEntityType();
		Class<? extends Attribute> attribute = attributeConditionSet.getAttribute();
		Object value = attributeConditionSet.getValue();
		AttributeCondition condition = attributeConditionSet.getAttributeCondition();

		IncludedEntities<Entity> envRefEntSetIncludedEnt = (IncludedEntities<Entity>) linkedElements.get(this.
				getEnvironmentReferencedEntities().getId());

		for (Entity entity : envRefEntSetIncludedEnt.getAllTargetEntities())
			if (entityType.isInstance(entity))
				try {
					Attribute attributeInstance = attribute.newInstance();
					attributeInstance.setEntity(entity);
					this.addRelationshipInstanceLinkedElement(queryIncludedEntities.getId(), attributeInstance);
				} catch (IllegalAccessException e) {
					Logger.getLogger(this.getClass()).error(e.getLocalizedMessage(), e);
				} catch (InstantiationException e) {
					Logger.getLogger(this.getClass()).error(e.getLocalizedMessage(), e);
				}

		ArrayList<Entity> newTargetEntities = new ArrayList<Entity>();
		for (ContextElement elem : linkedElements.values())
			if (attribute.isInstance(elem)) {
				Attribute attributeInstance = attribute.cast(elem);
				if (condition != null) {
					if (condition.resolve(attributeInstance))
						newTargetEntities.add(attributeInstance.getEntity());
				} else {
					ContextValue contextVal = attributeInstance.getValue();
					Object data = contextVal != null ? contextVal.getData() : null;
					if (data == value || data != null && data.equals(value))
						newTargetEntities.add(attributeInstance.getEntity());
				}
			}

		queryIncludedEntities.setAllTargetEntities(newTargetEntities);
	}
}
